(in-package #:tree-search)

(defun visit (function tree needle &key (test #'eql) key)
  "call function for every occurrence of needle in tree"
  (flet ((match ()
	   (let ((haystack (if key (funcall key tree)
			       tree)))
	     (funcall test haystack needle))))
    (when tree
      (if (match)
	  (funcall function tree))
      (if (listp tree)
	  (mapc (lambda (e)
		  (visit function e needle :test test :key key))
		tree)))
    nil))

(defun collect (tree needle &key (test #'eql) key)
  "collect all occurrences of needle from tree into a list"
  (let ((result nil))
    (visit
     (lambda (e)
       (push e result))
     tree needle :test test :key key)
    (nreverse result)))

(defun first-in-tree (tree needle &key (test #'eql) key)
  "find first occurrence of needle in tree"
  (block nil
    (visit
     (lambda (e)
       (return-from nil e))
     tree
     needle
     :test test
     :key key)))



