(in-package #:tree-search/test)

(use-package :tree-search)

(defvar *test-tree*
  '(this "is a"
    (large (tree with) (many elements))
    ("containing" numbers (like 4) or 5)))

(test first-in-tree
  "finding elements from a tree"
  (is (= 4
	 (first-in-tree *test-tree* 4)))
  (is (equal '(tree with)
	     (first-in-tree *test-tree* '(tree with) :test #'equal)))
  (is (equal "containing"
	     (first-in-tree *test-tree* "containing" :test #'equal)))
  (is (equal '(tree with)
	     (first-in-tree *test-tree* 2 :key (lambda (e)
						 (if (listp e) (length e))))))
  (is (equal '("containing" numbers (like 4) or 5)
	     (first-in-tree *test-tree* t :key (lambda (e)
						 (and
						  (listp e)
						  (some #'numberp e))))))
  (is (eq nil
	  (first-in-tree *test-tree* "something else"))))

(test visit
  "calling functions"
  (is (equal "45"
	     (with-output-to-string (out)
	       (visit (lambda (e) (write e :stream out))
		      *test-tree* t :key #'numberp))))
  (is (equal "containing"
	     (with-output-to-string (out)
	       (visit (lambda (e) (format out "~a" e))
		      *test-tree* "containing" :test #'equal))))
  (is (eq nil
	  (block nil
	    (visit (lambda (e) (declare (ignore e)) (return-from nil 'should-not-match))
		   *test-tree* "containing")))))

(test visit-returns-nil
  "returning nil from visit"
  (is (eq nil
	  (visit (constantly nil)
		 *test-tree* t))))

(test collect
  "collecting values into lists"
  (is (equal '(4 5)
	     (collect *test-tree* t :key #'numberp)))
  (is (equal '("is a" "containing")
	     (collect *test-tree* t :key #'stringp)))
  (is (equal '(this tree with many like)
	     (collect *test-tree* 4 :key (lambda (e)
					   (if (symbolp e)
					       (length (symbol-name e)))))))
  (is (equal '("containing")
	     (collect *test-tree* "containing" :test #'equal)))
  (is (eq nil
	  (collect *test-tree* "containing")))
  (is (eq nil
	  (collect *test-tree* 42))))
